///////////////////////////////////////////////////
// J1708Sample.cpp
// 720020-1187x
//
// Applied Data Systems
//
// Description
// -----------
// This sample application illustrates the use of 
// the J1708 serial driver on the AGX device.
//
// For more information on ADS' implementation of
// J1708 see ADS Specification 110025-1004x
//
// Requirements
// ------------
// The following registry entires must be added to 
// ADSLOAD.REG to enable the 1708 driver:
//
// [HKEY_LOCAL_MACHINE\Drivers\BuiltIn\Serial]
//        "Dll"="J1708.dll"
//        "FriendlyName"="SAE J1708 Interface"
//        "Index"=dword:1
//        "Prefix"="JSP"
//        "XmtMID"=dword:120
//        "ISTPriority"=dword:32
//        "RcvPktEventName"="J1708RcvPacketEvent"
//        "Tsp"=""
//        "DevConfig"=dword:0
//
// You must be using AGX Build 4.20.xx or later
//
// You must have an RS-232 to J1708 converter. ADS
// tested with the B&B Electronics model J1708P1
//
// History
// -------
// 720020-11871 - 1/11/05 (ctacke)
//   - Initial Release
//
///////////////////////////////////////////////////

// System inlcudes.
#include <windows.h>
#include <stdio.h>

// Project includes.
#include "J1708.h"

// My macros.
#define MSG_BUFFER_SIZE (J1708_MID_CHAR_SIZE + J1708_MAX_DATA_SIZE)

// Forward declarations.
//
void    ReadData(HANDLE hPort, HANDLE  hRcvPacketEvent);
void    SendData(HANDLE hPort, BOOL bSinglePacket);
DWORD   GetRandomNumber(void);
void    ToggleMessages(HANDLE hPort);
void    DisplayRegisters(void);
void    DisplayDriverStatus(HANDLE hPort);

// Global variables.
//
static BOOL bDriverMessages = FALSE;
static int  pktCntrs[] = {0,0,0};

// Main App
//
int WINAPI WinMain(
    HINSTANCE   hInst,
    HINSTANCE   hInstPrev,
    LPTSTR      lpszCmdLine,
    int         nCmdShow)
{
    BOOL    notFinished = TRUE;
    DWORD   dwError, dwActualLen;
    HANDLE  hPort;
    HANDLE  hRcvPacketEvent;
    UCHAR   keyPress;
    WCHAR   wszRcvPktEvtName[MAX_PATH];


	printf("\r\n*** J1708 Test Application ***\r\n");

    // Open the serial port.
    //
    hPort = CreateFile(TEXT("JSP1:"),                   // Pointer to the name of the port
                       GENERIC_READ | GENERIC_WRITE,    // Access (read-write) mode
                       0,                               // Share mode
                       NULL,                            // Not used
                       OPEN_EXISTING,                   // How to open the serial port
                       0,                               // Port attributes
                       NULL);                           // Not used

    if (hPort ==  INVALID_HANDLE_VALUE)
    {
        printf("JSP1: Open failed! (%d)\r\n", GetLastError());
        return(0);
    }

    // Seed the random number generator.
    //
    srand(GetTickCount());

    // Get the name of the packet reception event and create it.
    DeviceIoControl(hPort, IOCTL_J1708_GET_RCV_PKT_EVT_NAME, NULL, 0, wszRcvPktEvtName, MAX_PATH, &dwActualLen, NULL);

    if (dwActualLen == 0)
    {
        printf("Failed to get RX packet event name from driver!\r\n");
        return(0);
    }
    else if (!(hRcvPacketEvent = CreateEvent(NULL, FALSE, FALSE, wszRcvPktEvtName)))
    {
        dwError = GetLastError();
        printf("Cannot create named RX packet event: %s (%d)!\r\n", wszRcvPktEvtName, dwError);
        return(0);
    }

    do
    {
        printf("\r\nMENU\r\n----\r\n");
        printf("(1) Read packets\r\n");
        printf("(2) Send packets continuously\r\n");
        printf("(3) Send single packet\r\n");
        printf("(7) Toggle driver messages\r\n");
        printf("(8) Display driver status\r\n");
        printf("(0) Quit\r\n\n");

        // Wait for user to make a selection.
        //
		scanf("%s", &keyPress);

        switch (keyPress)
        {
            case '1':   ReadData(hPort, hRcvPacketEvent);
                        break;

            case '2':   SendData(hPort, FALSE);
                        break;

            case '3':   SendData(hPort, TRUE);
                        break;

            case '7':   ToggleMessages(hPort);
                        break;

            case '8':   DisplayDriverStatus(hPort);
                        break;

            case '0':   notFinished=FALSE;
                        break;
        }

    } while (notFinished);

    printf("*** J1708 Test Terminated. ***\r\n");
    CloseHandle(hPort);
    return(0);
}

void ReadData(HANDLE hPort, HANDLE  hRcvPacketEvent)
{
    char    msgBuffer[MSG_BUFFER_SIZE];
    char    cntrBuffer[6];
    DWORD   bytesRead;
    int     iPktCntr;

    printf("Reading data...\r\n");

    // Clear the receive buffer first.
    //
    PurgeComm(hPort, PURGE_RXCLEAR);

    // Reset packet counters.
    pktCntrs[0] = 0;
    pktCntrs[1] = 0;
    pktCntrs[2] = 0;

    do
    {
        WaitForSingleObject(hRcvPacketEvent, 1000);

        if (!(ReadFile(hPort, &msgBuffer, MSG_BUFFER_SIZE, &bytesRead, NULL)))
        {
            printf("Read Error %08X - A:%d B:%d C:%d\r\n", GetLastError(),
                                                                        pktCntrs[0],
                                                                        pktCntrs[1],
                                                                        pktCntrs[2]);
        }
        else if (bytesRead != 0)
        {
            // Get the packet number string from the message.
            memcpy(cntrBuffer, &msgBuffer[2], 5);
            cntrBuffer[5] = 0;
            // Convert it to an integer.
            iPktCntr = atoi(cntrBuffer);

            switch (msgBuffer[0])
            {
                case 'A':
                {
                    if ( iPktCntr != (pktCntrs[0] + 1) )
                    {
                        printf("Missed packet after (%d) from %c.\r\n", pktCntrs[0], msgBuffer[0]);
                    }
                    pktCntrs[0] = iPktCntr;
                    break;
                }
                case 'B':
                {
                    if ( iPktCntr != (pktCntrs[1] + 1) )
                    {
                        printf("Missed packet after (%d) from %c.\r\n", pktCntrs[1], msgBuffer[0]);
                    }
                    pktCntrs[1] = iPktCntr;
                    break;
                }
                case 'C':
                {
                    if ( iPktCntr != (pktCntrs[2] + 1) )
                    {
                        printf("Missed packet after (%d) from %c.\r\n", pktCntrs[2], msgBuffer[0]);
                    }
                    pktCntrs[2] = iPktCntr;
                    break;
                }
                default:
                {
                    printf("MID '%c' not recognized!\r\n", msgBuffer[0]);
                    break;
                }
            }
        }
    }
    while(!(scanf("%s")));  // wait for keypress to cancel

    printf("\r\n>> done. <<\n\r");
}

void SendData(HANDLE hPort, BOOL bSinglePacket)
{
    char    msgBuffer[J1708_MAX_DATA_SIZE + 1];
    DWORD   bytesSent, packetCounter;
    WCHAR   wideBuffer[J1708_MAX_DATA_SIZE + 1];

    printf("Sending data...\r\n");

    packetCounter = 0;
    do
    {
        sprintf(msgBuffer, ":%05d:J1708TstMsg.", ++packetCounter);
        if (bSinglePacket)
        {
            MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, msgBuffer, -1, wideBuffer, J1708_MAX_DATA_SIZE + 1);
            printf("\r\nSEND> %s", wideBuffer);
        }
        if (packetCounter >= 99999) packetCounter = 0;
        Sleep(GetRandomNumber());
        WriteFile(hPort, msgBuffer, strlen(msgBuffer), &bytesSent, NULL);
    }
    while((!(scanf("%c"))) && (!bSinglePacket));    // Wait for keypress or send just 1 packet.

    printf("\r\n>> done. <<\n\r");
}

DWORD GetRandomNumber(void)
{
    return( (((DWORD) rand()) & 0x3FF) + 25 );
}

void ToggleMessages(HANDLE hPort)
{
    switch (bDriverMessages)
    {
        case FALSE: bDriverMessages = TRUE;
                    DeviceIoControl(hPort, IOCTL_J1708_ENA_RTL_MSG, NULL, 0, NULL, 0, NULL, NULL);
                    break;

        case TRUE:  bDriverMessages = FALSE;
                    DeviceIoControl(hPort, IOCTL_J1708_DIS_RTL_MSG, NULL, 0, NULL, 0, NULL, NULL);
                    break;
    }
}

void DisplayDriverStatus(HANDLE hPort)
{
    J1708_DRVR_STS_STRUCT DriverStatus;

    DeviceIoControl(hPort, IOCTL_J1708_GET_DRIVER_STATUS, NULL, 0, (LPVOID) &DriverStatus, sizeof(LPVOID), NULL, NULL);

    printf("Driver Status:\r\n");
    printf(" - Word 1: %08X\r\n", DriverStatus.sts1);
    printf(" - Word 2: %08X\r\n", DriverStatus.sts2);
    printf(" - Word 3: %08X\r\n", DriverStatus.sts3);
}

